from typing import Dict

UNIT_MAP = {
    "time": "days",
    "year": "years",
    "dTime": "days",
    "enTemp": "GJ",
    "enProd": "GJ",
    "enInj": "GJ",
    "elecCost": "euro",
    "opex": "euro",
    "capex": "euro",
    "income": "euro",
    "cost": "euro",
    "grossRev": "euro",
    "tax": "euro",
    "netRev": "euro",
    "discRev": "euro",
    "npv": "euro",
    "lcoe_kWh": "euro",
    "lcoe_costs": "euro",
    "lcoe": "-",
    "injTemp": "C",
    "prodTemp": "C",
    "ipmt": "euro",
    "ppmt": "euro",
    "depreciation": "euro",
    "costTax": "euro",
    "grossRevTax": "euro",
    "enPower": "MW",
    "enPowercons": "MW",
    "COP": "-",
    "avgInjP": "bar",
}


WELL_PREFIX_UNIT: Dict[str, str] = {
    "temp_": "C",
    "wicum_": "m3",
    "winj_": "m3",
    "wiRate_": "m3/day",
    "wpcum_": "m3",
    "wprod_": "m3",
    "wpRate_": "m3/day",
    "dTemp_": "C",
    "enInj_": "GJ",
    "enTemp_": "GJ",
    "enProd_": "GJ",
    "dP_": "bar",
}
